import os.path

import pygame
from pygame.locals import *

class DataCache(object):
    _shared_state = {}
    
    def __init__(self, data_directory=""):
        self.__dict__ = self._shared_state
        if not hasattr(self, "initialized"):
            self.data_directory = data_directory
            self.image_cache = {}
            self.sound_cache = {}
            self.initialized = True
        
    def clear(self):
        self.image_cache = {}
        self.sound_cache = {}
        
    def get_image(self, filename):
        if filename in self.image_cache:
            return self.image_cache[filename]
        else:
            surface = pygame.image.load(os.path.join(self.data_directory, filename))
            surface = surface.convert_alpha()
            self.image_cache[filename] = surface
            return surface
    
    def get_sound(self, filename): 
        if filename in self.sound_cache:
            return self.sound_cache[filename]
        else:
            sound = pygame.mixer.Sound(os.path.join(self.data_directory, filename))
            self.sound_cache[filename] = sound
            return sound
        
    def get_data(self, filename):
        namespace = {}
        execfile(os.path.join(self.data_directory, filename), namespace)
        data = {}
        for name in namespace:
            if not name.startswith("_"):
                data[name] = namespace[name]
        return data