import math

import pygame
from pygame.locals import *

import game
from animation import Animation
from datacache import DataCache

class Entity(pygame.sprite.Sprite):
    
    def __init__(self, animations_name):
        pygame.sprite.Sprite.__init__(self)
        self.datacache = DataCache()
        animations = self.datacache.get_data("animations.py")[animations_name]
        self.animations = {}
        for name in animations:
            self.animations[name] = Animation(*animations[name])
        self.current_animation = self.animations["default"]
        self.facing = 0
        self.rect = self.image.get_rect()
        self.velocity_x = 0
        self.velocity_y = 0
        self.position_x = 0
        self.position_y = 0
        self.angular_velocity = 0
        self.level = None

    def get_image(self):
        surface = self.datacache.get_image(self.current_animation.get_frame())
        return pygame.transform.rotate(surface, self.facing)
    
    image = property(get_image)
    
    def set_state(self, position, facing):
        self.facing = facing
        self.position_x, self.position_y = position
        rect = self.image.get_rect()
        rect.center = position
        self.rect = rect
        
    def update(self, interval):
        self.current_animation.update(interval)
        velocity_x, velocity_y = self.velocity_x, self.velocity_y
        for disturbance in game.Game().level.disturbances:
            velocity_x, velocity_y = disturbance(velocity_x, velocity_y)
        new_position = (self.position_x + velocity_x * interval, 
                         self.position_y + velocity_y * interval)
        if self.angular_velocity:
            new_facing = self.facing + self.angular_velocity * interval
        elif velocity_x != 0 or velocity_y != 0:
            new_facing = math.degrees(math.atan2(-velocity_y, velocity_x))
        else:
            new_facing = self.facing
        try:
            self.level.move_entity(self, new_position, new_facing)
        except AttributeError:
            pass
    
    def collision(self, collisions):
        pass
    
    def distance(self, other):
        return ((self.position_x - other.position_x)**2 
                 + (self.position_y - other.position_y)**2)**0.5
                 
class Avatar(Entity):
    
    def update(self, interval):
        Entity.update(self, interval)
        if self.velocity_x != 0 or self.velocity_y != 0:
            if self.current_animation != self.animations["walk"]:
                self.current_animation = self.animations["walk"]
                self.current_animation.play(-1)
        else:
            self.current_animation = self.animations["default"]

class Device(Entity):

    def collision(self, collisions):
        try:
            for entity in collisions:
                if isinstance(entity, Tesseract):
                    entity.current_animation = entity.animations["visible"]
                    entity.current_animation.play(1, entity.destroy)
                    self.datacache.get_sound("pop.wav").play()
        except:
            pass
    
    def destroy(self):
        game.Game().level.remove_entity(self)


class Tesseract(Entity):
    
    def destroy(self):
        game.Game().next_level()
        
        