import pygame
from pygame.locals import *

from datacache import DataCache
from level import Level
import entity
from hud import HUD
from input import Input

class Game(object):
    _shared_state = {}
    
    def __init__(self):
        self.__dict__ = self._shared_state
        if not hasattr(self, "initialized"):
            pygame.init()
            self.datacache = DataCache()
            self.display = pygame.display.set_mode((800, 600))
            self.fullscreen = False
            self.input = Input()
            self.splash = None
            self.hud = None
            self.avatar = None
            self.level = None
            self.initialized = True
       
    def toggle_fullscreen(self):
        if self.fullscreen:
            self.display = pygame.display.set_mode((800, 600))
            pygame.mouse.set_visible(True)
            self.full_draw()
            self.fullscreen = False
        else:
            self.display = pygame.display.set_mode((800, 600), FULLSCREEN)
            pygame.mouse.set_visible(False)
            self.full_draw()
            self.fullscreen = True

    def set_splash(self, splash):
        self.hud = None
        self.avatar = None
        self.level = None
        self.splash = splash        

    def restart(self):
        self.time_limit = 480
        self.level_number = 1
        self.splash = None
        self.set_avatar("avatar")
        self.new_level()
        self.hud = HUD()
        self.full_draw()
        
    def new_level(self):
        self.level = Level(self.avatar)
        
    def next_level(self):
        self.level_number += 1
        if self.level_number == 11:
            pass
        else:
            self.new_level()
            self.full_draw()
    
    def set_avatar(self, avatar_name):
        avatar_data = self.datacache.get_data("entities.py")[avatar_name]
        self.avatar = entity.Avatar(*avatar_data)
        
    def update(self, interval):
        if self.splash:
            self.input.update()
        elif self.level_number == 11:
            self.set_splash("win_splash.png")
            self.datacache.get_sound("win.wav").play()
        elif self.time_limit <= 0:
            self.set_splash("lose_splash.png")
            self.datacache.get_sound("lose.wav").play()
        else:
            self.time_limit -= interval
            self.level.update(interval)
            self.input.update()
        
    def full_draw(self):
        if self.splash:
            splash = self.datacache.get_image(self.splash)
            pygame.display.get_surface().blit(splash, (0,0))
            pygame.display.update()
        else:
            self.level.full_draw()
            self.hud.full_draw()
     
    def draw(self):
        if self.splash:
            splash = self.datacache.get_image(self.splash)
            pygame.display.get_surface().blit(splash, (0,0))
            pygame.display.update()
        else:
            self.level.draw()
            self.hud.draw()
        