from __future__ import division

import pygame
from pygame.locals import *

from datacache import DataCache
import game

class HUD(object):
    
    def __init__(self):
        self.datacache = DataCache()
        self.background = "hud.png"
        self.font = pygame.font.Font(None, 36)
        self.proximity_sound = self.datacache.get_sound("proximity.wav")
        self.proximity_sound.set_volume(0)
        self.proximity_sound.play(-1)
        self.previous_seconds = 0
    
    def time(self):
        minutes = game.Game().time_limit // 60
        seconds = game.Game().time_limit - 60 * minutes
        time_string = "%i:%06.3f" % (minutes, seconds)
        if game.Game().time_limit > 15:
            return self.font.render(time_string, True, (0,0,0))
        else:
            if int(seconds) != int(self.previous_seconds):
                self.datacache.get_sound("beep.wav").play()
            self.previous_seconds = seconds
            return self.font.render(time_string, True, (255,0,0))
        
    def level(self):
        level_string = "Level %i" % game.Game().level_number
        return self.font.render(level_string, True, (0,0,0))
    
    def distance(self):
        distance = game.Game().avatar.distance(game.Game().level.tesseract)
        self.proximity_sound.set_volume(1 / distance * 96)
        distance_string = "%i m" % (distance // 24)
        if game.Game().avatar.distance(game.Game().level.tesseract) > 120:
            return self.font.render(distance_string, True, (0,0,0))
        else:
            return self.font.render(distance_string, True, (255,0,0))           

    def full_draw(self):
        display = pygame.display.get_surface()
        display.blit(self.datacache.get_image(self.background), (0,560))
        display.blit(self.time(), (128, 568))
        display.blit(self.level(), (353, 568))
        display.blit(self.distance(), (600, 568))
        pygame.display.update(Rect(0, 560, 800, 40))
        
    def draw(self):
        self.full_draw()