import sys
import math

import pygame
from pygame.locals import *

from datacache import DataCache
import game
import entity

class Input(object):   
    _shared_state = {}
    
    def __init__(self):
        self.__dict__ = self._shared_state
        if not hasattr(self, "initialized"):
            if pygame.joystick.get_count() > 0:
                self.joystick = pygame.joystick.Joystick(0)
                self.joystick.init()
            else:
                self.joystick = None
            self.bindings = {(KEYDOWN, K_q)    : self.quit,
                           (KEYDOWN, K_f)      : self.fullscreen,
                           (KEYDOWN, K_SPACE) : self.action,
                           (KEYDOWN, K_UP)   : self.up,
                           (KEYDOWN, K_DOWN) : self.down,
                           (KEYDOWN, K_LEFT) : self.left,
                           (KEYDOWN, K_RIGHT): self.right,
                           (KEYUP, K_UP)   : self.up,
                           (KEYUP, K_DOWN) : self.down,
                           (KEYUP, K_LEFT) : self.left,
                           (KEYUP, K_RIGHT):  self.right,
                           (JOYAXISMOTION, 0) : self.joyx,
                           (JOYAXISMOTION, 1) : self.joyy,
                           (JOYBUTTONDOWN, 0) : self.action}
            self.initialized = True
                           
    def update(self):   
        input_events = pygame.event.get() 
        for input_event in input_events:
            
            if input_event.type == KEYDOWN:
                callback = self.bindings.get((input_event.type, input_event.key), None)
                if callback:
                    callback(1)
                    
            if input_event.type == KEYUP:
                callback = self.bindings.get((input_event.type, input_event.key), None)
                if callback:
                    callback(-1)
                    
            if input_event.type == JOYAXISMOTION:
                callback = self.bindings.get((input_event.type, input_event.axis), None)   
                if callback:
                    callback(input_event.value)
                    
            if input_event.type == JOYBUTTONDOWN:
                callback = self.bindings.get((input_event.type, input_event.button), None)   
                if callback:
                    callback(1)
                    
            if input_event.type == VIDEOEXPOSE:
                game.Game().full_draw()
            
            if input_event.type == QUIT:
                self.quit(None)
                    
    def quit(self, value):
        sys.exit()
        
    def fullscreen(self, value):
        game.Game().toggle_fullscreen()
        
    def up(self, value):
        if game.Game().avatar: game.Game().avatar.velocity_y += -50 * value
        
    def down(self, value):
        if game.Game().avatar: game.Game().avatar.velocity_y += 50 * value
    
    def left(self, value):
        if game.Game().avatar: game.Game().avatar.velocity_x += -50 * value
    
    def right(self, value):
        if game.Game().avatar: game.Game().avatar.velocity_x += 50 * value
    
    def joyx(self, value):
        if game.Game().avatar: game.Game().avatar.velocity_x = 50 * value
    
    def joyy(self, value):
        if game.Game().avatar: game.Game().avatar.velocity_y = 50 * value
        
    def action(self, value):
        if game.Game().splash:
            game.Game().restart()
        else:
            avatar = game.Game().avatar
            facing = math.radians(avatar.facing)
            displacement_x = math.cos(facing) * 24
            displacement_y = -math.sin(facing) * 24
            device = entity.Device(*DataCache().get_data("entities.py")["device"])
            device.set_state((avatar.position_x + displacement_x, avatar.position_y + displacement_y), avatar.facing)
            game.Game().level.add_entity(device)
            device.current_animation = device.animations["activate"]
            device.current_animation.play(1, device.destroy)
            DataCache().get_sound("device.wav").play()